"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var events = require("events");
var fs = require("fs");
var utils = require("../helpers/Utils");
var ApiManager = require("../api/ApiManager");
var Layer = ApiManager.Layer;
var DynamicLayer = /** @class */ (function (_super) {
    __extends(DynamicLayer, _super);
    function DynamicLayer(manager, layerId, file, server, messageBus, connection) {
        var _this = _super.call(this) || this;
        _this.manager = manager;
        _this.layerId = layerId;
        _this.geojson = new Layer();
        _this.file = file;
        _this.server = server;
        _this.messageBus = messageBus;
        _this.connection = connection;
        return _this;
    }
    DynamicLayer.prototype.getLayer = function (req, res) {
        res.send(JSON.stringify(this.geojson));
    };
    DynamicLayer.prototype.OpenFile = function () {
        var _this = this;
        fs.readFile(this.file, 'utf8', function (err, data) {
            if (!err) {
                _this.geojson = JSON.parse(data);
                _this.geojson.features.forEach(function (f) {
                    _this.initFeature(f);
                });
            }
            else {
                console.log('error:' + _this.file);
            }
        });
    };
    DynamicLayer.prototype.getDataSource = function (req, res) {
        console.log('get DataSource');
        res.send(this.geojson);
    };
    DynamicLayer.prototype.initFeature = function (f) {
        if (!f.id) {
            f.id = utils.newGuid();
        }
    };
    DynamicLayer.prototype.updateSensorValue = function (ss, date, value) {
        ss.timestamps.push(date);
        ss.values.push(value);
        this.connection.updateSensorValue(ss.id, date, value);
    };
    DynamicLayer.prototype.addFeature = function (f, updated) {
        if (updated === void 0) { updated = true; }
        if (updated) {
            f.properties['updated'] = new Date().getTime();
        }
        this.initFeature(f);
        if (this.manager) {
            this.manager.addFeature(this.layerId, f, {}, function (cb) { });
        }
        //f.insertDate = new Date().getTime();
        //this.connection.updateFeature(this.layerId, f, 'feature-update');
    };
    DynamicLayer.prototype.start = function () {
        var _this = this;
        console.log('start case layer');
        this.server.get('/cases/' + this.layerId, function (req, res) { _this.getLayer(req, res); });
        this.startDate = new Date().getTime();
        //this.OpenFile();
        // this.connection.registerLayer(this.layerId, (action: string, msg: ClientConnection.LayerMessage, client: string) => {
        //     var feature;
        //     switch (action) {
        //         case 'logUpdate':
        //             // find feature
        //             var featureId = msg.object.featureId;
        //
        //             this.updateLog(featureId, msg.object, client, true);
        //             break;
        //         case 'featureUpdate':
        //             var ft: Feature = msg.object;
        //             this.updateFeature(ft, client, true);
        //             break;
        //     }
        // });
    };
    DynamicLayer.prototype.updateLog = function (featureId, msgBody, client, notify) {
        var f;
        console.log(JSON.stringify(msgBody));
        this.geojson.features.some(function (feature) {
            if (!feature.id || feature.id !== featureId) {
                return false;
            }
            // feature found
            f = feature;
            return true;
        });
        if (!f) {
            return; // feature not found
        }
        if (!f.hasOwnProperty('logs')) {
            f.logs = {};
        }
        if (!f.hasOwnProperty('properties')) {
            f.properties = {};
        }
        // apply changes
        var logs = msgBody.logs;
        for (var key in logs) {
            if (!f.logs.hasOwnProperty(key))
                f.logs[key] = [];
            logs[key].forEach(function (l) {
                f.logs[key].push(l);
                f.properties[key] = l.value;
            });
            // send them to other clients
            //this.connection.updateFeature(this.layerId, msgBody, 'logs-update', client);
        }
        console.log('Log update' + featureId);
        if (notify) {
            this.emit('featureUpdated', this.layerId, featureId);
        }
    };
    DynamicLayer.prototype.updateFeature = function (ft, client, notify) {
        this.initFeature(ft);
        var feature = this.geojson.features.filter(function (k) { return k.id && k.id === ft.id; });
        if (feature && feature.length > 0) {
            var index = this.geojson.features.indexOf(feature[0]);
            this.geojson.features[index] = ft;
        }
        else {
            this.geojson.features.push(ft);
        }
        //if (client)
        //this.connection.updateFeature(this.layerId, ft, 'feature-update', client);
        //else
        //this.connection.updateFeature(this.layerId, ft, 'feature-update');
        if (notify) {
            this.emit('featureUpdated', this.layerId, ft.id);
        }
    };
    return DynamicLayer;
}(events.EventEmitter));
exports.DynamicLayer = DynamicLayer;
//# sourceMappingURL=DynamicLayer.js.map